import { syntaxTree } from '@codemirror/language';
import { containsChild, isMatcher, walkBackward, getChildNodeText, createMatcher } from '../parser';
import { Order, SOrder, Limit, Slimit, Asc, Desc, In, NotIn, By, And, Or, DQL, EvalCallBody, ArgItem, Identifier, EqlSingle, StringLiteral, Number, FunctionCallBody, Pow, Mul, Div, Mod, Add, Sub, Eql, Gte, Gtr, Lte, Lss, InValue, AsExpr, Query, Namespace, IndexExpr, IndexItem, DataSourceList, DataSourceItem, Projection, ProjectionField, FieldName, FilterClause, FilterExpr, InFieldValue, BinaryExpr, DColon, Neq, DAnd, DOr, EqlRegex, NeqRegex, LimitClause, TimeClause, TimeRange, TimeExpr, ByClause, FieldListExpr, OrderClause, OffsetClause, SLimitClause, SOffsetClause, SOrderClause, OuterFunctionCallBody, ShowIdentifier, } from '@cloudcare/lezer-dql';
import { functionIdentifierTerms, matchOpTerms, binOpTerms, durationTerms, snippets, connectOpTerms } from './dql.terms';
const autocompleteNodes = {
    matchOp: matchOpTerms,
    connectOp: connectOpTerms,
    binOp: binOpTerms,
    duration: durationTerms,
    functionIdentifier: functionIdentifierTerms,
    number: [],
};
// ContextKind is the different possible value determinate by the autocompletion
export var ContextKind;
(function (ContextKind) {
    // dynamic autocompletion (required a distant server)
    ContextKind[ContextKind["Namespace"] = 0] = "Namespace";
    ContextKind[ContextKind["Index"] = 1] = "Index";
    ContextKind[ContextKind["DataSource"] = 2] = "DataSource";
    ContextKind[ContextKind["ShowFunction"] = 3] = "ShowFunction";
    ContextKind[ContextKind["FieldName"] = 4] = "FieldName";
    ContextKind[ContextKind["FieldValue"] = 5] = "FieldValue";
    // static autocompletion
    ContextKind[ContextKind["Function"] = 6] = "Function";
    ContextKind[ContextKind["OuterFunction"] = 7] = "OuterFunction";
    ContextKind[ContextKind["BinOp"] = 8] = "BinOp";
    ContextKind[ContextKind["MatchOp"] = 9] = "MatchOp";
    ContextKind[ContextKind["ConnectOp"] = 10] = "ConnectOp";
    ContextKind[ContextKind["Duration"] = 11] = "Duration";
    ContextKind[ContextKind["Offset"] = 12] = "Offset";
    ContextKind[ContextKind["SOffset"] = 13] = "SOffset";
    ContextKind[ContextKind["Limit"] = 14] = "Limit";
    ContextKind[ContextKind["SLimit"] = 15] = "SLimit";
    ContextKind[ContextKind["Orderby"] = 16] = "Orderby";
    ContextKind[ContextKind["SOrderby"] = 17] = "SOrderby";
    ContextKind[ContextKind["By"] = 18] = "By";
    ContextKind[ContextKind["Number"] = 19] = "Number";
    ContextKind[ContextKind["Desc"] = 20] = "Desc";
    ContextKind[ContextKind["Asc"] = 21] = "Asc";
})(ContextKind || (ContextKind = {}));
function trimQuotes(str) {
    // 1. 判断字符串首尾是否是同一个配对的引号
    if (!str)
        return str;
    // 支持 " ' `
    const first = str[0];
    const last = str[str.length - 1];
    if (first === last && `"'\``.includes(first)) {
        return str.slice(1, -1);
    }
    return str;
}
function computeStartCompleteWithQueted(node, pos) {
    // 有范围标志符号的 比如 {a,b,c}
    let start = node.from + 1;
    if (node.firstChild !== null) {
        start = pos;
    }
    return start;
}
function computeStartCompleteWithUnQueted(node, pos) {
    // 没有范围标志符号的 比如 a,b,c
    let start = node.from;
    if (node.firstChild !== null) {
        start = pos;
    }
    return start;
}
export function computeStartCompletePosition(state, node, pos) {
    var _a, _b, _c, _d, _e, _f, _g;
    let start = node.from;
    if ((node.type.id === Query && !containsChild(node, DataSourceList)) || node.type.id === DColon) {
        // L:: and not container L::humber
        start = pos;
    }
    else if (node.type.id === Number) {
        if (((_a = node.parent) === null || _a === void 0 ? void 0 : _a.type.id) === TimeExpr) {
            start = pos;
        }
    }
    else if (node.type.id === IndexExpr ||
        node.type.id === InValue ||
        node.type.id === Projection ||
        node.type.id === FilterClause ||
        node.type.id === InFieldValue) {
        // IndexExpr, InValue, Projection, FilterClause, InFieldValue
        // 这些节点的开始位置是从第一个子节点开始的
        start = computeStartCompleteWithQueted(node, pos);
    }
    else if (node.type.id === DataSourceList || node.type.id === FieldListExpr) {
        // DataSourceList
        // 这个节点的开始位置是从第一个子节点开始的
        start = computeStartCompleteWithUnQueted(node, pos);
    }
    else if ((node.type.id === OuterFunctionCallBody || node.type.id === EvalCallBody || node.type.id === FunctionCallBody) &&
        node.firstChild === null) {
        start++;
    }
    else if (node.type.id === StringLiteral &&
        (isMatcher(node) ||
            ((_b = node.parent) === null || _b === void 0 ? void 0 : _b.type.id) === IndexItem ||
            ((_c = node.parent) === null || _c === void 0 ? void 0 : _c.type.id) === DataSourceItem ||
            ((_d = node.parent) === null || _d === void 0 ? void 0 : _d.type.id) === InValue ||
            ((_e = node.parent) === null || _e === void 0 ? void 0 : _e.type.id) === AsExpr ||
            ((_f = node.parent) === null || _f === void 0 ? void 0 : _f.type.id) === FieldName ||
            ((_g = node.parent) === null || _g === void 0 ? void 0 : _g.type.id) === InFieldValue)) {
        start++;
    }
    else if (node.type.id === ByClause ||
        node.type.id === OrderClause ||
        node.type.id === SOrderClause ||
        node.type.id === LimitClause ||
        node.type.id === SLimitClause ||
        node.type.id === OffsetClause ||
        node.type.id === SOffsetClause) {
        start = pos;
    }
    const currentText = state.doc.slice(start, pos).toString();
    console.log('====currentText222', currentText);
    return start;
}
function arrayToCompletionResult(data, from, to, includeSnippet = false, span = true) {
    const options = data;
    if (includeSnippet) {
        options.push(...snippets);
    }
    return {
        from: from,
        to: to,
        options: options,
        validFor: span ? /^[a-zA-Z0-9_:]+$/ : undefined,
    };
}
function getNodeDocByExpr(tree, state, finder) {
    const currentNode = tree.getChild(finder);
    if (!currentNode) {
        return '';
    }
    return state.sliceDoc(currentNode.from, currentNode.to);
}
function getNamespaceByQuery(tree, state) {
    let currentNode = walkBackward(tree, Query);
    if (!currentNode) {
        // Weird case that shouldn't happen, because "VectorSelector" is by definition the parent of the LabelMatchers.
        return '';
    }
    let namespace = '';
    currentNode.cursor().iterate((node) => {
        // Continue until we find the VectorSelector, then look up the metric name.
        if (node.type.id === Namespace) {
            namespace = getNodeDocByExpr(node.node, state, Identifier);
            if (namespace) {
                return false;
            }
        }
    });
    return namespace;
}
function getIndexsByQuery(tree, state) {
    let currentNode = walkBackward(tree, Query);
    if (!currentNode) {
        // Weird case that shouldn't happen, because "VectorSelector" is by definition the parent of the LabelMatchers.
        return [];
    }
    let indexs = [];
    currentNode.cursor().iterate((node) => {
        // Continue until we find the VectorSelector, then look up the metric name.
        if (node.type.id === IndexExpr) {
            const IndexNodes = getChildNodeText(node.node, Identifier, StringLiteral);
            IndexNodes.forEach((indexNode) => {
                if (indexNode.type.id === StringLiteral) {
                    indexs.push(`${state.sliceDoc(indexNode.from, indexNode.to).slice(1, -1)}`);
                }
                else if (indexNode.type.id === Identifier) {
                    indexs.push(state.sliceDoc(indexNode.from, indexNode.to));
                }
            });
            return false;
        }
    });
    return indexs;
}
function getSourceListByQuery(tree, state) {
    let currentNode = walkBackward(tree, Query);
    if (!currentNode) {
        // Weird case that shouldn't happen, because "VectorSelector" is by definition the parent of the LabelMatchers.
        return [];
    }
    let sourcelist = [];
    currentNode.cursor().iterate((node) => {
        // Continue until we find the VectorSelector, then look up the metric name.
        if (node.type.id === DataSourceList) {
            const SourceNodes = getChildNodeText(node.node, Identifier, StringLiteral);
            SourceNodes.forEach((sourceNode) => {
                if (sourceNode.type.id === StringLiteral) {
                    sourcelist.push(`${state.sliceDoc(sourceNode.from, sourceNode.to).slice(1, -1)}`);
                }
                else if (sourceNode.type.id === Identifier) {
                    sourcelist.push(state.sliceDoc(sourceNode.from, sourceNode.to));
                }
            });
            return false;
        }
    });
    if (sourcelist.length === 0) {
        sourcelist = ['*'];
    }
    return sourcelist;
}
// analyzeCompletion is going to determinate what should be autocompleted.
// The value of the autocompletion is then calculate by the function buildCompletion.
// Note: this method is exported for testing purpose only. Do not use it directly.
export function analyzeCompletion(state, node, pos) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q;
    const result = [];
    const currentText = state.doc.slice(node.from, pos).toString();
    console.log(`currentText: ${currentText}`, 'nodeName: ', node.type.name, getNamespaceByQuery(node, state), getIndexsByQuery(node, state), getSourceListByQuery(node, state));
    switch (node.type.id) {
        case 0: {
            // 0 is the id of the error node
        }
        case DColon:
        case Query: {
            result.push({ kind: ContextKind.DataSource, Namespace: getNamespaceByQuery(node, state), Index: getIndexsByQuery(node, state) });
            break;
        }
        case DQL: {
            if (node.firstChild !== null && node.firstChild.type.id === 0) {
                // this situation can happen when there is nothing in the text area and the user is explicitly triggering the autocompletion (with ctrl + space)
                result.push({ kind: ContextKind.Namespace }, { kind: ContextKind.Function }, { kind: ContextKind.ShowFunction });
            }
            break;
        }
        case IndexExpr: {
            result.push({ kind: ContextKind.Index, Namespace: getNamespaceByQuery(node, state) });
            break;
        }
        case DataSourceList: {
            result.push({ kind: ContextKind.DataSource, Namespace: getNamespaceByQuery(node, state), Index: getIndexsByQuery(node, state) });
            break;
        }
        case Projection: {
            result.push({
                kind: ContextKind.FieldName,
                Namespace: getNamespaceByQuery(node, state),
                Index: getIndexsByQuery(node, state),
                DataSource: getSourceListByQuery(node, state),
            });
            break;
        }
        case FilterClause: {
            // 缺少 链接符的情况 {host = 'host' }
            //   if (node.firstChild) {
            //     result.push({
            //       kind: ContextKind.ConnectOp,
            //     });
            //   } else {
            //     result.push({
            //       kind: ContextKind.FieldName,
            //       Namespace: getNamespaceByQuery(node, state),
            //       Index: getIndexsByQuery(node, state),
            //       DataSource: getSourceListByQuery(node, state),
            //     });
            //   }
            break;
        }
        case InFieldValue: {
            break;
        }
        case TimeClause: {
            result.push({ kind: ContextKind.Number });
            break;
        }
        case TimeRange: {
            result.push({ kind: ContextKind.Number });
            break;
        }
        case FieldListExpr: {
            result.push({
                kind: ContextKind.FieldName,
                Namespace: getNamespaceByQuery(node, state),
                Index: getIndexsByQuery(node, state),
                DataSource: getSourceListByQuery(node, state),
            });
            break;
        }
        case Identifier: {
            // Identifier is the root node of the DQL language
            // It can be used to complete the namespace, index, datasource, fieldname, fieldvalue
            if (((_a = node.parent) === null || _a === void 0 ? void 0 : _a.type.id) === Namespace) {
                const queryNode = walkBackward(node, Query);
                if (queryNode === null || queryNode === void 0 ? void 0 : queryNode.getChildren(DColon).length) {
                    //
                    result.push({
                        kind: ContextKind.Namespace,
                    });
                }
                else {
                    result.push({
                        kind: ContextKind.Namespace,
                    }, {
                        kind: ContextKind.ShowFunction,
                    }, {
                        kind: ContextKind.OuterFunction,
                    });
                }
            }
            else if (((_b = node.parent) === null || _b === void 0 ? void 0 : _b.type.id) === FieldName && ((_d = (_c = node.parent) === null || _c === void 0 ? void 0 : _c.parent) === null || _d === void 0 ? void 0 : _d.type.id) === ProjectionField) {
                // L::xxx:(avg(), host)
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                }, { kind: ContextKind.Function });
            }
            else if (((_e = node.parent) === null || _e === void 0 ? void 0 : _e.type.id) === ArgItem) {
                //Avg(xxx)
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                });
            }
            else if (((_f = node.parent) === null || _f === void 0 ? void 0 : _f.type.id) === IndexItem) {
                result.push({
                    kind: ContextKind.Index,
                    Namespace: getNamespaceByQuery(node, state),
                });
            }
            else if (((_g = node.parent) === null || _g === void 0 ? void 0 : _g.type.id) === DataSourceItem) {
                result.push({
                    kind: ContextKind.DataSource,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                });
            }
            else if (((_h = node.parent) === null || _h === void 0 ? void 0 : _h.type.id) === FieldName) {
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                });
            }
            else if (((_j = node.parent) === null || _j === void 0 ? void 0 : _j.type.id) === FilterExpr) {
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                });
            }
            break;
        }
        case StringLiteral: {
            if (((_k = node.parent) === null || _k === void 0 ? void 0 : _k.type.id) === IndexItem) {
                result.push({
                    kind: ContextKind.Index,
                    Namespace: getNamespaceByQuery(node, state),
                });
            }
            else if (((_l = node.parent) === null || _l === void 0 ? void 0 : _l.type.id) === DataSourceItem) {
                result.push({
                    kind: ContextKind.DataSource,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                });
            }
            else if (((_m = node.parent) === null || _m === void 0 ? void 0 : _m.type.id) === InFieldValue) {
                if (isMatcher(node)) {
                    result.push({
                        kind: ContextKind.FieldValue,
                        Namespace: getNamespaceByQuery(node, state),
                        Index: getIndexsByQuery(node, state),
                        DataSource: getSourceListByQuery(node, state),
                        matcher: createMatcher(node, state),
                    });
                }
            }
            else if (((_o = node.parent) === null || _o === void 0 ? void 0 : _o.type.id) === FieldName) {
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                });
            }
            else if (((_p = node.parent) === null || _p === void 0 ? void 0 : _p.type.id) === FilterExpr) {
                if (isMatcher(node)) {
                    result.push({
                        kind: ContextKind.FieldValue,
                        Namespace: getNamespaceByQuery(node, state),
                        Index: getIndexsByQuery(node, state),
                        DataSource: getSourceListByQuery(node, state),
                        matcher: createMatcher(node, state),
                    });
                }
                else {
                    result.push({
                        kind: ContextKind.FieldName,
                        Namespace: getNamespaceByQuery(node, state),
                        Index: getIndexsByQuery(node, state),
                        DataSource: getSourceListByQuery(node, state),
                    });
                }
            }
            break;
        }
        case Number: {
            if (((_q = node.parent) === null || _q === void 0 ? void 0 : _q.type.id) === TimeExpr) {
                result.push({
                    kind: ContextKind.Duration,
                });
            }
            else {
                result.push({
                    kind: ContextKind.Number,
                });
            }
            break;
        }
        case ShowIdentifier: {
            result.push({
                kind: ContextKind.ShowFunction,
            });
            break;
        }
        case EvalCallBody:
        case FunctionCallBody: {
            break;
        }
        case OuterFunctionCallBody: {
            //   result.push({
            //     kind: ContextKind.Function,
            //   });
            break;
        }
        case InValue: {
            break;
        }
        case EqlSingle:
        case Neq:
        case EqlRegex:
        case NeqRegex:
        case In:
        case NotIn: {
            result.push({ kind: ContextKind.MatchOp });
            break;
        }
        case By:
            break;
        case OrderClause: {
            if (!node.getChildren(By).length) {
                result.push({ kind: ContextKind.By });
            }
            else {
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                });
            }
            break;
        }
        case Order: {
            result.push({ kind: ContextKind.Orderby }, { kind: ContextKind.By });
            break;
        }
        case SOrderClause: {
            if (!node.getChildren(By).length) {
                result.push({ kind: ContextKind.By });
            }
            else {
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                });
            }
            break;
        }
        case SOrder:
            result.push({ kind: ContextKind.SOrderby }, { kind: ContextKind.By });
            break;
        case Limit:
            result.push({ kind: ContextKind.Limit });
            break;
        case Slimit:
            result.push({ kind: ContextKind.SLimit });
            break;
        case Asc:
            result.push({ kind: ContextKind.Asc });
            break;
        case Desc:
            result.push({ kind: ContextKind.Desc });
            break;
        case And:
        case DAnd:
        case Or:
        case DOr: {
            result.push({ kind: ContextKind.ConnectOp });
            break;
        }
        case Pow:
        case Mul:
        case Div:
        case Mod:
        case Add:
        case Sub:
        case Eql:
        case Gte:
        case Gtr:
        case Lte:
        case Lss:
            result.push({ kind: ContextKind.BinOp });
            break;
        case BinaryExpr:
            //       if ()
            //   result.push({ kind: ContextKind.BinOp });
            break;
    }
    return result;
}
export class HybridComplete {
    constructor(dqlClient) {
        this.dqlClient = dqlClient;
    }
    DQL(context) {
        const { state, pos } = context;
        const tree = syntaxTree(state).resolve(pos, -1);
        // The lines above can help you to print the current lezer tree.
        // It's useful when you are trying to understand why it doesn't autocomplete.
        // console.log(syntaxTree(state).topNode.toString());
        // console.log(`current node: ${tree.type.name}`);
        const contexts = analyzeCompletion(state, tree, pos);
        let asyncResult = Promise.resolve([]);
        let completeSnippet = false;
        let span = true;
        for (const context of contexts) {
            switch (context.kind) {
                case ContextKind.Namespace: {
                    asyncResult = asyncResult.then((result) => {
                        return this.autocompleteNamespace(result, context);
                    });
                    break;
                }
                case ContextKind.Index: {
                    asyncResult = asyncResult.then((result) => {
                        return this.autocompleteIndex(result, context);
                    });
                    break;
                }
                case ContextKind.DataSource: {
                    asyncResult = asyncResult.then((result) => {
                        return this.autocompleteDataSource(result, context);
                    });
                    break;
                }
                case ContextKind.ShowFunction: {
                    asyncResult = asyncResult.then((result) => {
                        return this.autocompleteShowFuncs(result, context);
                    });
                    break;
                }
                case ContextKind.FieldName: {
                    asyncResult = asyncResult.then((result) => {
                        return this.autocompleteFieldNames(result, context);
                    });
                    break;
                }
                case ContextKind.FieldValue: {
                    asyncResult = asyncResult.then((result) => {
                        return this.autocomplteFieldValues(result, context);
                    });
                    break;
                }
                case ContextKind.OuterFunction: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat(autocompleteNodes.functionIdentifier);
                    });
                    break;
                }
                case ContextKind.Function: {
                    completeSnippet = true;
                    asyncResult = asyncResult.then((result) => {
                        return result.concat(autocompleteNodes.functionIdentifier);
                    });
                    break;
                }
                case ContextKind.ConnectOp: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat(autocompleteNodes.connectOp);
                    });
                    break;
                }
                case ContextKind.BinOp: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat(autocompleteNodes.binOp);
                    });
                    break;
                }
                case ContextKind.MatchOp: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat(autocompleteNodes.matchOp);
                    });
                    break;
                }
                case ContextKind.Duration: {
                    span = false;
                    asyncResult = asyncResult.then((result) => {
                        return result.concat(autocompleteNodes.duration);
                    });
                    break;
                }
                case ContextKind.Offset: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat([{ label: 'offset' }]);
                    });
                    break;
                }
                case ContextKind.SOffset: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat([{ label: 'soffset' }]);
                    });
                    break;
                }
                case ContextKind.Limit: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat([{ label: 'limit' }]);
                    });
                    break;
                }
                case ContextKind.SLimit: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat([{ label: 'slimit' }]);
                    });
                    break;
                }
                case ContextKind.Orderby: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat([{ label: 'order by' }]);
                    });
                    break;
                }
                case ContextKind.SOrderby: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat([{ label: 'sorder by' }]);
                    });
                    break;
                }
                case ContextKind.By: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat([{ label: 'by' }]);
                    });
                    break;
                }
                case ContextKind.Number: {
                    break;
                }
                case ContextKind.Asc: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat([{ label: 'asc' }]);
                    });
                    break;
                }
                case ContextKind.Desc: {
                    asyncResult = asyncResult.then((result) => {
                        return result.concat([{ label: 'desc' }]);
                    });
                    break;
                }
            }
        }
        return asyncResult.then((result) => {
            return arrayToCompletionResult(result, computeStartCompletePosition(state, tree, pos), pos, completeSnippet, span);
        });
    }
    autocompleteNamespace(result, context) {
        if (!this.dqlClient)
            return result;
        const namespaceCompetion = [];
        return this.dqlClient.namespace().then((namespaces) => {
            namespaces.forEach((namespace) => {
                namespaceCompetion.push({
                    label: namespace.name,
                    info: namespace.help,
                    type: 'constant',
                });
            });
            return namespaceCompetion;
        });
    }
    autocompleteIndex(result, context) {
        if (!this.dqlClient)
            return result;
        const indexCompetion = [];
        return this.dqlClient.indexValues(context.Namespace).then((indexs) => {
            indexs.forEach((index) => {
                indexCompetion.push({
                    label: index,
                    type: 'keyword',
                });
            });
            return indexCompetion;
        });
    }
    autocompleteDataSource(result, context) {
        if (!this.dqlClient)
            return result;
        const datasourceCompetion = [];
        return this.dqlClient.datasourceValues(context.Namespace, context.Index).then((datasources) => {
            datasources.forEach((datasource) => {
                datasourceCompetion.push({
                    label: datasource,
                    type: 'keyword',
                });
            });
            return datasourceCompetion;
        });
    }
    autocompleteShowFuncs(result, context) {
        if (!this.dqlClient)
            return result;
        const showFuncCompetion = [];
        return this.dqlClient.showFunctions().then((showfuns) => {
            showfuns.forEach((showfun) => {
                showFuncCompetion.push({
                    label: showfun.name,
                    type: 'function',
                    detail: 'function',
                    info: showfun.info,
                });
            });
            return showFuncCompetion;
        });
    }
    autocompleteFieldNames(result, context) {
        if (!this.dqlClient)
            return result;
        const fieldNamesCompetion = [];
        return this.dqlClient.fieldNames(context.Namespace, context.Index, context.DataSource).then((fieldNames) => {
            fieldNames.forEach((fieldName) => {
                fieldNamesCompetion.push({
                    label: fieldName.name,
                    detail: fieldName.type,
                    type: 'keyword',
                    info: fieldName.info,
                });
            });
            return fieldNamesCompetion;
        });
    }
    autocomplteFieldValues(result, context) {
        if (!this.dqlClient)
            return result;
        const fieldValuesCompetion = [];
        return this.dqlClient.fieldValues(context.Namespace, context.Index, context.DataSource, context.matcher).then((fieldValues) => {
            fieldValues.forEach((fieldValue) => {
                fieldValuesCompetion.push({
                    label: fieldValue,
                    type: 'text',
                });
            });
            return fieldValuesCompetion;
        });
    }
}
//# sourceMappingURL=hybrid.js.map